"""
hudai.resources.company
"""
from ..helpers.resource import Resource


class CompanyResource(Resource):
    def __init__(self, client):
        Resource.__init__(self, client, base_path='/companies')
        self.resource_name = 'Company'

    def list(self, name=None, ticker=None, key_term=None, page=None):
        return self._list(name=name,
                          key_term=key_term,
                          ticker=ticker,
                          page=page)

    def create(self, name=None, ticker=None):
        return self._create(name=name, ticker=ticker)

    def fetch(self, entity_id):
        return self._fetch(entity_id)

    def update(self, entity_id, name=None, ticker=None):
        return self._update(entity_id, name=name, ticker=ticker)

    def delete(self, entity_id):
        return self._delete(entity_id)

    def domains(self, entity_id):
        return self.http_get('/{id}/domains',
                             params={'id': entity_id})

    def key_terms(self, entity_id):
        return self.http_get('/{id}/key-terms',
                             params={'id': entity_id})



"""
hudai.resources.relevant_articles
"""
from ..helpers.resource import Resource


class RelevantArticlesResource(Resource):
    def __init__(self, client):
        Resource.__init__(self, client, base_path='/articles/relevant')
        self.resource_name = 'RelevantArticles'

    def list(self,
             user_id=None,
             article_id=None,
             published_before=None,
             published_after=None,
             scored_above=None,
             scored_below=None,
             scored_before=None,
             scored_after=None,
             page=None):
        return self._list(
            user_id=user_id,
            article_id=article_id,
            published_before=published_before,
            published_after=published_after,
            scored_above=scored_above,
            scored_below=scored_below,
            scored_before=scored_before,
            scored_after=scored_after,
            page=page
        )

    def create(self,
               user_id=None,
               article_id=None,
               score=None,
               scored_at=None,
               article_published_at=None):
        return self._create(
            user_id=user_id,
            article_id=article_id,
            score=score,
            scored_at=scored_at,
            article_published_at=article_published_at
        )

    def fetch(self, entity_id):
        return self._fetch(entity_id)

    def delete(self, entity_id):
        return self._delete(entity_id)



"""
hudai.client
"""

from datetime import datetime, timedelta
import requests

from .error import HudAiError
from .resources import *
from .helpers.http_client import HttpClient


class Client(object):
    """
    API Client for HUD.ai that handles the API token injection and translation
    to/from Python objects
    """
    def __init__(self,
                 client_id=None,
                 client_secret=None,
                 base_url='https://api.hud.ai/v1',
                 auth_url='https://accounts.hud.ai',
                 redirect_uri=None,
                 debug=False):
        if not client_id:
            raise HudAiError('missing client_id', 'initialization_error')

        self._debug = debug

        self._client_id = client_id
        self._client_secret = client_secret
        self._auth_url = auth_url
        self._redirect_uri = redirect_uri

        self._auth_code = None
        self.access_token = None
        self.refresh_token = None
        self.token_expires_at = None

        self._http_client = HttpClient(self, base_url)

        self.article_highlights = ArticleHighlightsResource(self._http_client)
        self.article_key_terms = ArticleKeyTermResource(self._http_client)
        self.article_tags = ArticleTagResource(self._http_client)
        self.articles = ArticleResource(self._http_client)
        self.collateral = CollateralResource(self._http_client)
        self.company_key_terms = CompanyKeyTermResource(self._http_client)
        self.companies = CompanyResource(self._http_client)
        self.domains = DomainResource(self._http_client)
        self.key_terms = KeyTermResource(self._http_client)
        self.organizations = OrganizationResource(self._http_client)
        self.people = PersonResource(self._http_client)
        self.person_key_terms = PersonKeyTermResource(self._http_client)
        self.person_quotes = PersonQuoteResource(self._http_client)
        self.relevant_article_collateral = RelevantArticleCollateralResource(self._http_client)
        self.relevant_articles = RelevantArticlesResource(self._http_client)
        self.system_events = SystemEventResource(self._http_client)
        self.system_tasks = SystemTaskResource(self._http_client)
        self.text_corpora = TextCorpusResource(self._http_client)
        self.user_companies = UserCompanyResource(self._http_client)
        self.user_contacts = UserContactResource(self._http_client)
        self.user_digest_subscriptions = UserDigestSubscriptionResource(self._http_client)
        self.user_key_terms = UserKeyTermResource(self._http_client)
        self.users = UserResource(self._http_client)

        # Preserve backwards compatibility
        self._add_deprecated_attributes()

    def get_authorize_uri(self):
        """
        Builds authorization URL to send users to to grant access and return a
        `code` to the redirect_uri
        """
        if not self._redirect_uri:
            raise HudAiError('cannot generate authorization uri without redirect_uri')

        return (
            '{host}/oauth2/authorize' +
            '?response_type=code' +
            '&client_id={client_id}' +
            '&redirect_uri={redirect_uri}'
        ).format(
            host=self._auth_url,
            client_id=self._client_id,
            redirect_uri=self._redirect_uri
        )

    def refresh_tokens(self):
        """
        Refreshes the tokens (access and refresh) if required
        """
        if self.token_expires_at and self.token_expires_at > datetime.now():
            return

        if self._auth_code:
            return self._exchange_auth_code()

        if self.refresh_token:
            return self._refresh_tokens()

        if self._client_secret:
            return self._exchange_client_credentials()

    def set_auth_code(self, code):
        """
        Store the authorization code given back from the auth server for future
        exchange to get a user access token
        """
        self._auth_code = code

    # Private

    def _add_deprecated_attributes(self):
        self.article_highlight = self.article_highlights
        self.article_key_term = self.article_key_terms
        self.article = self.articles
        self.company_key_term = self.company_key_terms
        self.company = self.companies
        self.domain = self.domains
        self.key_term = self.key_terms
        self.relevant_article = self.relevant_articles
        self.system_event = self.system_events
        self.system_task = self.system_tasks
        self.text_corpus = self.text_corpora
        self.user_company = self.user_companies
        self.user_digest_subscription = self.user_digest_subscriptions
        self.user_key_term = self.user_key_terms
        self.user = self.users

    def _get_tokens(self, data):
        token_url = '{host}/oauth2/token'.format(host=self._auth_url)

        response = requests.post(token_url, json=data).json()

        self.access_token = response.get('access_token')

        if self._debug and not self.access_token:
            print "hudai.client error:get_token", response

        self.refresh_token = response.get('refresh_token', None)

        expires_in = response.get('expires_in', 0)
        self.token_expires_at = datetime.now() + timedelta(milliseconds=expires_in)

    def _exchange_auth_code(self):
        self._get_tokens({
            'grant_type':    'authorization_code',
            'client_id':     self._client_id,
            'client_secret': self._client_secret,
            'code':          self._auth_code,
            'redirect_uri':  self._redirect_uri,
        })

    def _exchange_client_credentials(self):
        self._get_tokens({
            'grant_type':    'client_credentials',
            'client_id':     self._client_id,
            'client_secret': self._client_secret,
        })

    def _refresh_tokens(self):
        self._get_tokens({
            'grant_type':    'refresh_grant',
            'client_id':     self._client_id,
            'client_secret': self._client_secret,
            'refresh_token': self.refresh_token,
        })

HudAi = Client



"""
hudai.helpers.resource
"""

from pydash import omit, omit_by, is_none

from ..error import HudAiError


class Resource(object):
    """
    Inheritable class to build out the actions that can be taken at a particular
    endpoint (e.g. /my-resource)
    Includes:
        - all standard HTTP verbs as via `http_VERB` e.g. `http_get`
        - helper methods for the standard CRUD actions
    """
    def __init__(self, client, base_path=''):
        """
        :param client: API client
        """

        if client is None:
            raise HudAiError('client required', 'initialization_error')

        self._client = client
        self._base_path = base_path


    # Standard HTTP Verbs with url params injected into the given paths


    def http_get(self, path, **request_params):
        """
        Wrapped HTTP action that
            - builds off the base path
            - pulls apart the query params from the URL params
        """
        full_path = self._build_path(path, request_params.get('params'))

        client_params = omit(request_params, 'params')

        return self._client.http_get(full_path, **client_params)


    def http_post(self, path, **request_params):
        """
        Wrapped HTTP action that
            - builds off the base path
            - pulls apart the query params from the URL params
        """
        full_path = self._build_path(path, request_params.get('params'))

        client_params = omit(request_params, 'params')

        if client_params.get('data'):
            client_params['data'] = omit_by(client_params.get('data'), is_none)

        return self._client.http_post(full_path, **client_params)


    def http_put(self, path, **request_params):
        """
        Wrapped HTTP action that
            - builds off the base path
            - pulls apart the query params from the URL params
        """
        full_path = self._build_path(path, request_params.get('params'))

        client_params = omit(request_params, 'params')

        if client_params.get('data'):
            client_params['data'] = omit_by(client_params.get('data'), is_none)

        return self._client.http_put(full_path, **client_params)


    def http_patch(self, path, **request_params):
        """
        Wrapped HTTP action that
            - builds off the base path
            - pulls apart the query params from the URL params
        """
        full_path = self._build_path(path, request_params.get('params'))

        client_params = omit(request_params, 'params')

        if client_params.get('data'):
            client_params['data'] = omit_by(client_params.get('data'), is_none)

        return self._client.http_patch(full_path, **client_params)


    def http_delete(self, path, **request_params):
        """
        Wrapped HTTP action that
            - builds off the base path
            - pulls apart the query params from the URL params
        """
        full_path = self._build_path(path, request_params.get('params'))

        client_params = omit(request_params, 'params')

        return self._client.http_delete(full_path, **client_params)


    # CRUD actions common to many endpoints


    def _list(self, **query_params):
        query_params = self._set_limit_offset(query_params)
        return self.http_get('/', query_params=query_params)

    def _create(self, **data):
        return self.http_post('/', data=data)

    def _fetch(self, entity_id):
        return self.http_get('/{id}', params={'id': entity_id})

    def _update(self, entity_id, **data):
        return self.http_put('/{id}', params={'id': entity_id}, data=data)

    def _delete(self, entity_id):
        return self.http_delete('/{id}', params={'id': entity_id})


    # Helper functions


    def _build_path(self, url, query_params):
        """
        Build the url path string
        :return url:
        """
        path = "{}{}".format(self._base_path, url)

        if not query_params:
            return path

        return path.format(**query_params)

    def _set_limit_offset(self, params):
        params['limit'] = 50

        if 'page' in params:
            page = max(params['page'], 0)
            params['offset'] = 50 * page
            del params['page']
        else:
            params['offset'] = 0

        return params




"""
hudai.resources.company_key_term
"""
from ..helpers.resource import Resource


class CompanyKeyTermResource(Resource):
    def __init__(self, client):
        Resource.__init__(
            self, client, base_path='/companies/{company_id}/key-terms')
        self.resource_name = 'CompanyKeyTerm'

    def list(self, company_id, page=None):
        query_params = self._set_limit_offset({'page': page})

        return self.http_get('/', params={'company_id': company_id},
                             query_params=query_params)

    def create(self, company_id, term):
        return self.http_post('/', params={'company_id': company_id},
                              data={'term': term})

    def fetch(self, company_id, term):
        return self.http_get('/{term}',
                             params={'company_id': company_id, 'term': term})

    def delete(self, company_id, term):
        return self.http_delete('/{term}',
                                params={'company_id': company_id, 'term': term})



"""
hudai.resources.user
"""
from ..helpers.resource import Resource


class UserResource(Resource):
    def __init__(self, client):
        Resource.__init__(self, client, base_path='/users')
        self.resource_name = 'User'

    def list(self,
             email=None,
             digest_subscription_day=None,
             digest_subscription_hour=None,
             name=None,
             key_term=None,
             company_id=None,
             page=None):
        return self._list(
            email=email,
            digest_subscription_day=digest_subscription_day,
            digest_subscription_hour=digest_subscription_hour,
            name=name,
            key_term=key_term,
            company_id=company_id,
            page=page
        )

    def create(self, email=None, name=None, time_zone=None):
        return self._create(email=email, name=name, time_zone=time_zone)

    def fetch(self, entity_id):
        return self._fetch(entity_id)

    def me(self):
        return self.http_get('/me')

    def update(self, entity_id, email=None, name=None, time_zone=None):
        return self._update(entity_id,
                            email=email,
                            name=name,
                            time_zone=time_zone)

    def delete(self, entity_id):
        return self._delete(entity_id)



"""
hudai.resources.user_digest_subscription
"""
from ..helpers.resource import Resource


class UserDigestSubscriptionResource(Resource):
    def __init__(self, client):
        Resource.__init__(self, client, base_path='/users/digest-subscriptions')
        self.resource_name = 'UserDigestSubscription'

    def list(self, user_id, day_of_week=None, iso_hour=None, page=None):
        query_params = self._set_limit_offset({
            'day_of_week': day_of_week,
            'iso_hour': iso_hour,
            'page': page,
            'user_id': user_id
        })

        return self.http_get('/', query_params=query_params)

    def create(self, user_id, day_of_week=None, iso_hour=None):
        return self.http_post('/',
                              data={'day_of_week': day_of_week,
                                    'iso_hour': iso_hour,
                                    'user_id': user_id})

    def fetch(self, user_id, digest_id):
        return self.http_get('/{id}',
                             query_params={'user_id': user_id},
                             params={'id': digest_id})

    def delete(self, user_id, digest_id):
        return self.http_delete('/{id}',
                                query_params={'user_id': user_id},
                                params={'id': digest_id})



"""
hudai.resources.article_highlights
"""
from ..helpers.resource import Resource


class ArticleHighlightsResource(Resource):
    def __init__(self, client):
        Resource.__init__(self, client, base_path='/article-highlights')
        self.resource_name = 'ArticleHighlights'

    def list(self, article_id=None, link_hash=None, user_id=None):
        return self._list(article_id=article_id,
                          link_hash=link_hash,
                          user_id=user_id)

    def create(self, article_id=None, user_id=None, body=None):
        return self._create(article_id=article_id, user_id=user_id, body=body)

    def fetch(self, entity_id):
        return self._fetch(entity_id)

    def update(self, entity_id, body=None):
        return self._update(entity_id, body=body)

    def delete(self, entity_id):
        return self._delete(entity_id)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from os import path
from setuptools import setup, find_packages


def read(filename):
    return open(path.join(path.dirname(__file__), filename)).read()

def file_lines(filename):
    return read(filename).strip().split('\n')

def parse_requirements(filename):
    return [line.strip() for line in file_lines(filename) if line.strip()]


pkg = {}
for line in file_lines('hudai/__init__.py'):
    if line.startswith('__'):
        exec(line, pkg)

setup(
    name=pkg['__package_name__'],
    version=pkg['__version__'],
    url=pkg['__url__'],
    license=pkg['__license__'],
    author=pkg['__author__'],
    author_email=pkg['__email__'],
    description=pkg['__description__'],
    packages=find_packages(exclude=['*.test', '*.test.*', 'test.*', 'test']),
    install_requires=parse_requirements('requirements.txt'),
    download_url='{}/archive/{}.tar.gz'.format(pkg['__url__'], pkg['__version__']),
    keywords=pkg['__keywords__'],
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Intended Audience :: Developers',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Topic :: Software Development :: Build Tools',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.7'
    ],
)



"""
hudai.resources.domain
"""
from ..helpers.resource import Resource


class DomainResource(Resource):
    def __init__(self, client):
        Resource.__init__(
            self, client, base_path='/companies/domains')
        self.resource_name = 'Domain'

    def list(self, company_id, page=None):
        query_params = self._set_limit_offset({'page': page})
        query_params['company_id'] = company_id

        return self.http_get('/', query_params=query_params)

    def create(self, company_id, hostname):
        return self.http_post('/', data={'hostname': hostname,'company_id': company_id})

    def fetch(self, entity_id):
        return self.http_get('/{id}',
                             params={'id': entity_id})

    def delete(self, entity_id):
        return self.http_delete('/{id}',
                                params={'id': entity_id})
